#!/usr/bin/env python3
# -*- coding: utf8 -*-

import os
import re
import sys
from pathlib import Path
import csv
import json
from dataclasses import dataclass
from datetime import datetime
import locale
from typing import List
from enum import Enum
from pronounceable import PronounceableWord, generate_word
from chromepdf import ChromePDF
from jinja2 import Environment, FileSystemLoader, select_autoescape
from config import config

#print(config.tao.host)

class MetaDataTestEnum(Enum):
    ATTENDANCE = 'E0'
    DIGITAL = 'E1'
    ENGLISH = 'E4'

class MetaDataPeriodEnum(Enum):
    AM = 'AM'
    PM = 'PM'

@dataclass
class FileMetaData:
    test: MetaDataTestEnum  # 'E0' -> presence | 'E1' -> QCM numerique | 'E4' -> QCM anglais
    date: str # YYYY-MM-DD
    period: MetaDataPeriodEnum # 'AM' | 'PM'

@dataclass
class Student:
    fullname: str
    birth_date: str
    scei: int
    password: str

@dataclass
class Test:
    title: str
    date: str
    timeslot: str
    duration: str


generate_password = lambda : PronounceableWord().length(8, 12)

CSV_DELIMITERS = [ ',', ';' ]  # we need to detect delimiters since they are different in the provided CSV

def load_students_from_csv_file(csv_filename : str) -> List[Student]:
    students = []
    with open(csv_filename, newline='', encoding='cp1252') as csvfile:
        # try to detect CSV delimiter
        detected_delimiter = None
        header = csvfile.readline()
        for potential_delimiter in CSV_DELIMITERS:
            if header.count(potential_delimiter) == 2:
                detected_delimiter = potential_delimiter
                break
        if detected_delimiter is None:
            raise Exception('Failed to detect CSV delimiter.')

        student_reader = csv.reader(csvfile, delimiter=detected_delimiter, quotechar='|')
        students = [ Student(row[0], row[1], row[2], generate_password()) for row in student_reader ]
    return students

def save_students_as_json_file(json_filename : str, students : List[Student]):
    with open(json_filename, 'w') as outfile:
        json.dump(students, outfile, default=lambda o: o.__dict__, indent=2, ensure_ascii=False)

def get_metadata_from_filename(filename : str) -> FileMetaData:
    basename = os.path.basename(filename)
    name = os.path.splitext(basename)[0]

    # 'epreuve-E0_2019-06-28_ADA_ADB_AM'
    m = re.match(r"epreuve-(E[014])_(\d\d\d\d-\d\d-\d\d)_ADA_ADB_(AM|PM)", name)
    if not m:
        raise Exception(f'Invalid filename format (expecting: <epreuve-E?_YYYY-MM-DD_ADA_ADB_?M> but got <{name}>)')
    else:
        test = MetaDataTestEnum(m[1])
        date = datetime.strptime(m[2], '%Y-%m-%d') #.strftime('%d %B %Y')
        period = MetaDataPeriodEnum(m[3])

        return FileMetaData(test, date, period)

def format_test(metadata : FileMetaData):
    formatted_test_title = {
        MetaDataTestEnum.ATTENDANCE: 'Présence',
        MetaDataTestEnum.DIGITAL: 'QCM Numérique',
        MetaDataTestEnum.ENGLISH: 'QCM Anglais'
    }.get(metadata.test)

    saved_locale = locale.getlocale(locale.LC_ALL)
    locale.setlocale(locale.LC_ALL, 'fr_FR')
    french_formatted_date = metadata.date.strftime('%d %B %Y')
    locale.setlocale(locale.LC_ALL, saved_locale)

    return { 'title': formatted_test_title, 
             'date': french_formatted_date,
             'duration': '30 min',
             'timeslot': '10:00 - 10:30' if metadata.period != MetaDataPeriodEnum.AM else '14:00 - 14:30'
           }

def convert_html_to_pdf(html_content : str, output_filename : str):
    # PATH_TO_CHROME_EXE = '/usr/bin/google-chrome-stable'
    PATH_TO_CHROME_EXE = '/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome'
    cpdf = ChromePDF(PATH_TO_CHROME_EXE)
    with open(os.path.abspath(output_filename),'w') as output_file:
        cpdf.html_to_pdf(html_content, output_file)

def generate_html_tickets(test, students : List[Student]) -> str:
    env = Environment(
        loader = FileSystemLoader('../templates'),
        autoescape = select_autoescape(['html', 'xml'])
    )
    template = env.get_template('stickers.tmpl')

    return template.render(test=test, students=students)


# File format: Nom;DateNaissance;SCEI 
# File encoding: cp1252
CSV_FILE = 'epreuve-E4_2021-03-31_ADA_ADB_AM.csv'
CSV_DIR = '../sample_data'
base_filename = os.path.splitext(CSV_FILE)[0]

metadata = get_metadata_from_filename(base_filename)
students = load_students_from_csv_file(CSV_DIR + '/' + CSV_FILE)

# test = Test(title='Anglais', date='31 Mars 2021', timeslot='10:00 - 10:30', duration='30 min')
# student = Student(fullname='HARTMANN MOSCISKI Quinten', birth_date='1978-03-19', scei='54642', password='tbandtshe')

# File encoding: utf8
OUTPUT_DIR = '../output'
Path(OUTPUT_DIR).mkdir(parents=False, exist_ok=True)

save_students_as_json_file(f'{OUTPUT_DIR}/{base_filename}.json', students)

html_content = generate_html_tickets(format_test(metadata), students)

# save as .html
with open(f'{OUTPUT_DIR}/{base_filename}.html', 'w') as output_html_file:
    output_html_file.write(html_content)

# convert as .pdf
convert_html_to_pdf(html_content, f'{OUTPUT_DIR}/{base_filename}.pdf')
